@@ -1,26 +1,29 @@
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
 import os
 import sys
 import warnings
 
-from pylint import __pkginfo__, config, extensions, interfaces
-from pylint.constants import full_version
+from pylint import __pkginfo__, extensions, interfaces
+from pylint.config.config_initialization import _config_initialization
+from pylint.constants import DEFAULT_PYLINT_HOME, OLD_DEFAULT_PYLINT_HOME, full_version
 from pylint.lint.pylinter import PyLinter
 from pylint.lint.utils import ArgumentPreprocessingError, preprocess_options
-from pylint.utils import utils
+from pylint.utils import print_full_documentation, utils
 
 try:
     import multiprocessing
+    from multiprocessing import synchronize  # noqa pylint: disable=unused-import
 except ImportError:
-    multiprocessing = None  # type: ignore
+    multiprocessing = None  # type: ignore[assignment]
 
 
 def _cpu_count() -> int:
     """Use sched_affinity if available for virtualized or containerized environments."""
     sched_getaffinity = getattr(os, "sched_getaffinity", None)
-    # pylint: disable=not-callable,using-constant-test
+    # pylint: disable=not-callable,using-constant-test,useless-suppression
     if sched_getaffinity:
         return len(sched_getaffinity(0))
     if multiprocessing:
@@ -29,7 +32,7 @@ def _cpu_count() -> int:
 
 
 def cb_list_extensions(option, optname, value, parser):
-    """List all the extensions under pylint.extensions"""
+    """List all the extensions under pylint.extensions."""
 
     for filename in os.listdir(os.path.dirname(extensions.__file__)):
         if filename.endswith(".py") and not filename.startswith("_"):
@@ -40,12 +43,12 @@ def cb_list_extensions(option, optname, value, parser):
 
 def cb_list_confidence_levels(option, optname, value, parser):
     for level in interfaces.CONFIDENCE_LEVELS:
-        print("%-18s: %s" % level)
+        print(f"%-18s: {level}")
     sys.exit(0)
 
 
 def cb_init_hook(optname, value):
-    """exec arbitrary code to set sys.path for instance"""
+    """Execute arbitrary code to set 'sys.path' for instance."""
     exec(value)  # pylint: disable=exec-used
 
 
@@ -53,10 +56,7 @@ UNUSED_PARAM_SENTINEL = object()
 
 
 class Run:
-    """helper class to use as main for pylint :
-
-    run(*sys.argv[1:])
-    """
+    """Helper class to use as main for pylint with 'run(*sys.argv[1:])'."""
 
     LinterClass = PyLinter
     option_groups = (
@@ -92,6 +92,7 @@ group are mutually exclusive.",
                     "init-hook": (cb_init_hook, True),
                     "rcfile": (self.cb_set_rcfile, True),
                     "load-plugins": (self.cb_add_plugins, True),
+                    "enable-all-extensions": (self.cb_enable_all_extensions, False),
                     "verbose": (self.cb_verbose_mode, False),
                     "output": (self.cb_set_output, True),
                 },
@@ -156,7 +157,8 @@ group are mutually exclusive.",
                         "callback": self.cb_list_messages,
                         "group": "Commands",
                         "level": 1,
-                        "help": "Generate pylint's messages.",
+                        "help": "Display a list of all pylint's messages divided by whether "
+                        "they are emittable with the given interpreter.",
                     },
                 ),
                 (
@@ -167,8 +169,8 @@ group are mutually exclusive.",
                         "callback": self.cb_list_messages_enabled,
                         "group": "Commands",
                         "level": 1,
-                        "help": "Display a list of what messages are enabled "
-                        "and disabled with the given configuration.",
+                        "help": "Display a list of what messages are enabled, "
+                        "disabled and non-emittable with the given configuration.",
                     },
                 ),
                 (
@@ -246,16 +248,6 @@ group are mutually exclusive.",
                         "displayed, and no reports are done by default.",
                     },
                 ),
-                (
-                    "py3k",
-                    {
-                        "action": "callback",
-                        "callback": self.cb_python3_porting_mode,
-                        "help": "In Python 3 porting mode, all checkers will be "
-                        "disabled and only messages emitted by the porting "
-                        "checker will be displayed.",
-                    },
-                ),
                 (
                     "verbose",
                     {
@@ -266,6 +258,15 @@ group are mutually exclusive.",
                         "will be displayed.",
                     },
                 ),
+                (
+                    "enable-all-extensions",
+                    {
+                        "action": "callback",
+                        "callback": self.cb_enable_all_extensions,
+                        "help": "Load and enable all available extensions. "
+                        "Use --list-extensions to see a list all available extensions.",
+                    },
+                ),
             ),
             option_groups=self.option_groups,
             pylintrc=self._rcfile,
@@ -278,7 +279,20 @@ group are mutually exclusive.",
         # load command line plugins
         linter.load_plugin_modules(self._plugins)
         # add some help section
-        linter.add_help_section("Environment variables", config.ENV_HELP, level=1)
+        linter.add_help_section(
+            "Environment variables",
+            f"""
+The following environment variables are used:
+    * PYLINTHOME
+    Path to the directory where persistent data for the run will be stored. If
+not found, it defaults to '{DEFAULT_PYLINT_HOME}' or '{OLD_DEFAULT_PYLINT_HOME}'
+(in the current working directory).
+    * PYLINTRC
+    Path to the configuration file. See the documentation for the method used
+to search for configuration file.
+""",
+            level=1,
+        )
         linter.add_help_section(
             "Output",
             "Using the default text output, the message format is :                          \n"
@@ -312,45 +326,12 @@ group are mutually exclusive.",
         # read configuration
         linter.disable("I")
         linter.enable("c-extension-no-member")
-        try:
-            linter.read_config_file(verbose=self.verbose)
-        except OSError as ex:
-            print(ex, file=sys.stderr)
-            sys.exit(32)
 
-        config_parser = linter.cfgfile_parser
-        # run init hook, if present, before loading plugins
-        if config_parser.has_option("MASTER", "init-hook"):
-            cb_init_hook(
-                "init-hook", utils._unquote(config_parser.get("MASTER", "init-hook"))
-            )
-        # is there some additional plugins in the file configuration, in
-        if config_parser.has_option("MASTER", "load-plugins"):
-            plugins = utils._splitstrip(config_parser.get("MASTER", "load-plugins"))
-            linter.load_plugin_modules(plugins)
-        # now we can load file config and command line, plugins (which can
-        # provide options) have been registered
-        linter.load_config_file()
-
-        if reporter:
-            # if a custom reporter is provided as argument, it may be overridden
-            # by file parameters, so re-set it here, but before command line
-            # parsing so it's still overrideable by command line option
-            linter.set_reporter(reporter)
-        try:
-            args = linter.load_command_line_configuration(args)
-        except SystemExit as exc:
-            if exc.code == 2:  # bad options
-                exc.code = 32
-            raise
-        if not args:
-            print(linter.help())
-            sys.exit(32)
+        args = _config_initialization(linter, args, reporter, verbose_mode=self.verbose)
 
         if linter.config.jobs < 0:
             print(
-                "Jobs number (%d) should be greater than or equal to 0"
-                % linter.config.jobs,
+                f"Jobs number ({linter.config.jobs}) should be greater than or equal to 0",
                 file=sys.stderr,
             )
             sys.exit(32)
@@ -364,17 +345,10 @@ group are mutually exclusive.",
             elif linter.config.jobs == 0:
                 linter.config.jobs = _cpu_count()
 
-        # We have loaded configuration from config file and command line. Now, we can
-        # load plugin specific configuration.
-        linter.load_plugin_configuration()
-
-        # Now that plugins are loaded, get list of all fail_on messages, and enable them
-        linter.enable_fail_on_messages()
-
         if self._output:
             try:
-                with open(self._output, "w") as output:
-                    linter.reporter.set_output(output)
+                with open(self._output, "w", encoding="utf-8") as output:
+                    linter.reporter.out = output
                     linter.check(args)
                     score_value = linter.generate_reports()
             except OSError as ex:
@@ -398,29 +372,36 @@ group are mutually exclusive.",
                 # We need to make sure we return a failing exit code in this case.
                 # So we use self.linter.msg_status if that is non-zero, otherwise we just return 1.
                 sys.exit(self.linter.msg_status or 1)
-            elif score_value is not None and score_value >= linter.config.fail_under:
-                sys.exit(0)
+            elif score_value is not None:
+                if score_value >= linter.config.fail_under:
+                    sys.exit(0)
+                else:
+                    # We need to make sure we return a failing exit code in this case.
+                    # So we use self.linter.msg_status if that is non-zero, otherwise we just return 1.
+                    sys.exit(self.linter.msg_status or 1)
             else:
                 sys.exit(self.linter.msg_status)
 
     def version_asked(self, _, __):
-        """callback for version (i.e. before option parsing)"""
+        """Callback for version (i.e. before option parsing)."""
         self._version_asked = True
 
     def cb_set_rcfile(self, name, value):
-        """callback for option preprocessing (i.e. before option parsing)"""
+        """Callback for option preprocessing (i.e. before option parsing)."""
         self._rcfile = value
 
     def cb_set_output(self, name, value):
-        """callback for option preprocessing (i.e. before option parsing)"""
+        """Callback for option preprocessing (i.e. before option parsing)."""
         self._output = value
 
     def cb_add_plugins(self, name, value):
-        """callback for option preprocessing (i.e. before option parsing)"""
+        """Callback for option preprocessing (i.e. before option parsing)."""
         self._plugins.extend(utils._splitstrip(value))
 
     def cb_error_mode(self, *args, **kwargs):
-        """error mode:
+        """Callback for --errors-only.
+
+        Error mode:
         * disable all but error messages
         * disable the 'miscellaneous' checker which can be safely deactivated in
           debug
@@ -430,37 +411,37 @@ group are mutually exclusive.",
         self.linter.error_mode()
 
     def cb_generate_config(self, *args, **kwargs):
-        """optik callback for sample config file generation"""
+        """Optik callback for sample config file generation."""
         self.linter.generate_config(skipsections=("COMMANDS",))
         sys.exit(0)
 
     def cb_generate_manpage(self, *args, **kwargs):
-        """optik callback for sample config file generation"""
+        """Optik callback for sample config file generation."""
         self.linter.generate_manpage(__pkginfo__)
         sys.exit(0)
 
     def cb_help_message(self, option, optname, value, parser):
-        """optik callback for printing some help about a particular message"""
+        """Optik callback for printing some help about a particular message."""
         self.linter.msgs_store.help_message(utils._splitstrip(value))
         sys.exit(0)
 
     def cb_full_documentation(self, option, optname, value, parser):
-        """optik callback for printing full documentation"""
-        self.linter.print_full_documentation()
+        """Optik callback for printing full documentation."""
+        print_full_documentation(self.linter)
         sys.exit(0)
 
     def cb_list_messages(self, option, optname, value, parser):
-        """optik callback for printing available messages"""
+        """Optik callback for printing available messages."""
         self.linter.msgs_store.list_messages()
         sys.exit(0)
 
     def cb_list_messages_enabled(self, option, optname, value, parser):
-        """optik callback for printing available messages"""
+        """Optik callback for printing available messages."""
         self.linter.list_messages_enabled()
         sys.exit(0)
 
     def cb_list_groups(self, *args, **kwargs):
-        """List all the check groups that pylint knows about
+        """List all the check groups that pylint knows about.
 
         These should be useful to know what check groups someone can disable
         or enable.
@@ -469,9 +450,13 @@ group are mutually exclusive.",
             print(check)
         sys.exit(0)
 
-    def cb_python3_porting_mode(self, *args, **kwargs):
-        """Activate only the python3 porting checker."""
-        self.linter.python3_porting_mode()
-
     def cb_verbose_mode(self, *args, **kwargs):
         self.verbose = True
+
+    def cb_enable_all_extensions(self, option_name: str, value: None) -> None:
+        """Callback to load and enable all available extensions."""
+        for filename in os.listdir(os.path.dirname(extensions.__file__)):
+            if filename.endswith(".py") and not filename.startswith("_"):
+                extension_name = f"pylint.extensions.{filename[:-3]}"
+                if extension_name not in self._plugins:
+                    self._plugins.append(extension_name)
